package server

import (
	"fmt"
	"net/http"
	"strconv"
	//	"io"
	"caj-larsson/bog/dataswamp"
	"caj-larsson/bog/dataswamp/namespace"
	"caj-larsson/bog/dataswamp/swampfile"
	fs_swampfile "caj-larsson/bog/infrastructure/fs/swampfile"
	sql_namespace "caj-larsson/bog/infrastructure/sqlite/namespace"
)

type Router interface {
	HandleFunc(pattern string, handler func(http.ResponseWriter, *http.Request))
	ServeHTTP(http.ResponseWriter, *http.Request)
}

type Bog struct {
	router       Router
	file_service dataswamp.SwampFileService
	address      string
}

func buildFileDataRepository(config FileConfig) swampfile.Repository {
	r, err := fs_swampfile.NewRepository(config.Path)
	if err != nil {
		panic(err)
	}
	return r
}

func buildUserAgentRepository(config DatabaseConfig) namespace.Repository {
	if config.Backend != "sqlite" {
		panic("Can only handle sqlite")
	}
	return sql_namespace.NewRepository(config.Connection)
}

func (b *Bog) fileHandler(w http.ResponseWriter, r *http.Request) {
	if r.URL.Path == "/" {
		fmt.Fprintf(w, "Hi")
		return
	}

	ref := swampfile.FileReference{r.URL.Path, r.Header["User-Agent"][0]}

	switch r.Method {
	case "GET":
		swamp_file, err := b.file_service.OpenOutFile(ref)

		if err == swampfile.ErrNotExists {
			http.NotFound(w, r)
			return
		}

		if err != nil {
			panic(err)
		}

		http.ServeContent(w, r, swamp_file.Path(), swamp_file.Modified(), swamp_file)
	case "POST":
		fallthrough
	case "PUT":
		size_str := r.Header["Content-Length"][0]

		size, err := strconv.ParseInt(size_str, 10, 64)
		if err != nil {
			w.WriteHeader(422)
			return
		}

		err = b.file_service.SaveFile(ref, r.Body, size)

		if err != nil {
			panic(err)
		}
	}
	return
}

func (b *Bog) routes() {
	b.router.HandleFunc("/", b.fileHandler)
}

func New(config *Configuration) *Bog {
	b := new(Bog)
	b.address = config.bindAddress()

	fsSwampRepo := buildFileDataRepository(config.File)
	uaRepo := buildUserAgentRepository(config.Database)

	b.file_service = dataswamp.NewSwampFileService(
		uaRepo, fsSwampRepo, config.Quota.ParsedSizeBytes(), config.Quota.ParsedDuration(),
	)

	b.router = new(http.ServeMux)
	b.routes()
	return b
}

func (b *Bog) Run() {
	http.ListenAndServe(b.address, b.router)
}
